#!/bin/bash

set -eu

## {{ 

fatal() {
	echo "$@" >&2
	exit 1
}

needpackages() {
	local got_all="true"
	
	for pkg in "$@" ; do
		if ! which "$pkg" >/dev/null 2>/dev/null ; then
			echo "Missing package '$pkg'" >&2
			got_all="false"
		fi
	done
	
	if [[ "$got_all" == "false" ]] ; then
		exit 1
	else
		return 0
	fi
}

runbat() {
	local VCPLATFORM="${PB_PLATFORM}"
	if [[ "$PB_PLATFORM" == "x64" ]] ; then
		VCPLATFORM=amd64
	fi
	
	local TMPFILE=$(mktemp --suffix=.cmd)
	
	echo '@echo off' > "$TMPFILE"
	echo "call \"C:\\Program Files (x86)\\Microsoft Visual Studio ${PB_VC_VERSION}.0\\VC\\vcvarsall.bat\" ${VCPLATFORM}" >> "$TMPFILE"
	cat >> "$TMPFILE"
	
	#unix2dos
	sed -i 's/$/\r/' "$TMPFILE"
	
	PATH=/cygdrive/c/WINDOWS/system32 cmd /c "$(cygpath -w "$TMPFILE")"
	
	rm "$TMPFILE"
}

fastrmdir() {
	local tmpdir=$(mktemp -d)
	mv "$1" "$tmpdir"
	rm -r "$tmpdir" &
}

cachedownload() {
	if [[ ! -f "./.cache/$1" ]] ; then
		wget "$2" -O "./.cache/$1"
	fi
}

## }} 

if [[ ! -f ./php-build.configuration.sh ]] ; then
	fatal "Configuration file not found!"
fi

. ./php-build.configuration.sh

PB_DEPSFILE="deps-${PB_PHP_MAJOR_MINOR}-vc${PB_DEPS_VC_VERSION}-${PB_PLATFORM}.7z"
PB_BASEDIR="/cygdrive/c/php-sdk/phpdev/vc${PB_VC_VERSION}/${PB_PLATFORM}"
PB_BASEDIR_W32="C:\\php-sdk\\phpdev\\vc${PB_VC_VERSION}\\${PB_PLATFORM}"

needpackages wget unzip p7zip

if [[ "$#" > 0 ]] ; then

	if [[ "$1" == "--cmd" ]] ; then
		runbat <<EOD
cd C:\php-sdk
call bin\phpsdk_setvars.bat
cd ${PB_BASEDIR_W32}\src
start cmd
EOD
		exit $?
		
		
	elif [[ "$1" == "--explorer" ]] ; then
		cygstart explorer "${PB_BASEDIR_W32}\\src"
		exit 0
		
	elif [[ "$1" == "--force" ]] ; then
		:
		
	elif [[ "$1" == "--help" ]] ; then
		cat <<EOD
Usage:
  php-build.sh [options]

Options:
  --cmd             Open cmd.exe terminal in src dir with correct environment
  --explorer        Open explorer in src dir with correct environment
  --force           Delete php-sdk directory if it already exists
  --help            Display this message

Examples:
  - Build PHP, but fail if destination directory already exists:
    ./php-build.sh    
  
  - Build PHP, removing destination directory if it already exists:
    ./php-build.sh --force
EOD
		exit 1
	
	else
		fatal "Unrecognised argument '$1'"
		
	fi
fi

if [[ -d "/cygdrive/c/php-sdk/" ]] ; then

	if [[ "$#" > 0 && "$1" == "--force" ]] ; then
		echo "Removing existing C:\\php-sdk directory..."		
		fastrmdir /cygdrive/c/php-sdk/
	else
		fatal $' C:\\php-sdk\\ directory already exists!\n Please either move/remove or run with --force.'
	fi
fi

echo "Retrieving source files..."

if [[ ! -d ./.cache ]] ; then
	mkdir ./.cache
fi

cachedownload "php-sdk-binary-tools-20110915.zip" "http://windows.php.net/downloads/php-sdk/php-sdk-binary-tools-20110915.zip"
cachedownload "$PB_DEPSFILE" "http://windows.php.net/downloads/php-sdk/${PB_DEPSFILE}"
cachedownload "${PB_PHP_GIT_BRANCH}.zip" "https://github.com/php/php-src/archive/${PB_PHP_GIT_BRANCH}.zip"

if [[ "$PB_VC_VERSION" -lt 12 ]] ; then
	cachedownload "jom_1_0_14.zip" "http://download.qt.io/official_releases/jom/jom_1_0_14.zip"
fi

echo "Unpacking binary-tools..."

mkdir /cygdrive/c/php-sdk/
unzip -q ./.cache/php-sdk-binary-tools-20110915.zip -d /cygdrive/c/php-sdk
if [[ "$PB_VC_VERSION" -lt 12 ]] ; then
	unzip ./.cache/jom_1_0_14.zip jom.exe -d /cygdrive/c/php-sdk/bin/
fi
chmod -R 755 /cygdrive/c/php-sdk/bin

echo "Preparing source tree..."

runbat <<'EOD'
cd C:\php-sdk\
call bin\phpsdk_setvars.bat
call bin\phpsdk_buildtree.bat phpdev

EOD
if [[ ! -d "/cygdrive/c/php-sdk/phpdev/vc${PB_VC_VERSION}" ]] ; then
	cp -R /cygdrive/c/php-sdk/phpdev/vc9 "/cygdrive/c/php-sdk/phpdev/vc${PB_VC_VERSION}"
fi

echo "Unpacking php source..."

if [[ -d "php-src-${PB_PHP_GIT_BRANCH}" ]] ; then
	fastrmdir "php-src-${PB_PHP_GIT_BRANCH}"
fi

unzip -q "./.cache/${PB_PHP_GIT_BRANCH}.zip" -d "${PB_BASEDIR}"
mv "${PB_BASEDIR}/php-src-${PB_PHP_GIT_BRANCH}" "${PB_BASEDIR}/src"

echo "Unpacking dependencies..."

FULLPATH_TO_DEPSFILE_ZIP="$(realpath ./.cache/$PB_DEPSFILE)"
pushd "${PB_BASEDIR}" >/dev/null
7z x "$FULLPATH_TO_DEPSFILE_ZIP" >/dev/null
chmod -R a+rwX deps
popd >/dev/null

echo "Building..."

runbat <<EOD
cd C:\php-sdk
call bin\phpsdk_setvars.bat
cd ${PB_BASEDIR_W32}\src

call buildconf
call configure --help
call configure $PB_CONFIGURE

EOD

# Hooks

post_configure_pre_nmake

# Build 

NMAKE=nmake
if [[ "$PB_VC_VERSION" -lt 12 ]] ; then
	NMAKE=jom
fi

runbat <<EOD
cd C:\php-sdk
call bin\phpsdk_setvars.bat
cd ${PB_BASEDIR_W32}\src

$NMAKE
EOD

# Hooks

post_nmake_pre_snap

# nmake snap

runbat <<EOD
cd C:\php-sdk
call bin\phpsdk_setvars.bat
cd ${PB_BASEDIR_W32}\src

$NMAKE snap
EOD

echo ""
echo "The compiled PHP is now under"
echo -n "  ${PB_BASEDIR_W32}\\src\\"
sed -r -n 's~^BUILD_DIR=(.*)~\1~p' "${PB_BASEDIR}/src/Makefile"
echo ""
